#!/usr/bin/env python

__description__ = 'VBA project stream plugin for oledump.py'
__author__ = 'Didier Stevens'
__version__ = '0.0.5'
__date__ = '2022/05/02'

"""

Source code put in public domain by Didier Stevens, no Copyright
https://DidierStevens.com
Use at your own risk

History:
  2020/07/16: start
  2020/07/17: added Hashcat hash
  2020/07/20: added option -w
  2020/07/21: refactor
  2020/08/15: 0.0.3 bin bugfix
  2022/05/02: 0.0.4 added support for plaintext passwords
  2022/05/03: 0.0.5 added decoding of CMG and GC

Todo:
"""

# https://interoperability.blob.core.windows.net/files/MS-OVBA/%5bMS-OVBA%5d.pdf

def GetDictionary(passwordfile):
    if passwordfile != '.':
        return File2Strings(passwordfile)
    else:
# https://github.com/magnumripper/JohnTheRipper/blob/bleeding-jumbo/run/password.lst
        return [
          'infected',
          'P@ssw0rd',
          'VelvetSweatshop',
          '123456',
          '12345',
          'password',
          'password1',
          '123456789',
          '12345678',
          '1234567890',
          'abc123',
          'computer',
          'tigger',
          '1234',
          'qwerty',
          'money',
          'carmen',
          'mickey',
          'secret',
          'summer',
          'internet',
          'a1b2c3',
          '123',
          'service',
          'canada',
          'hello',
          'ranger',
          'shadow',
          'baseball',
          'donald',
          'harley',
          'hockey',
          'letmein',
          'maggie',
          'mike',
          'mustang',
          'snoopy',
          'buster',
          'dragon',
          'jordan',
          'michael',
          'michelle',
          'mindy',
          'patrick',
          '123abc',
          'andrew',
          'bear',
          'calvin',
          'changeme',
          'diamond',
          'fuckme',
          'fuckyou',
          'matthew',
          'miller',
          'tiger',
          'trustno1',
          'alex',
          'apple',
          'avalon',
          'brandy',
          'chelsea',
          'coffee',
          'falcon',
          'freedom',
          'gandalf',
          'green',
          'helpme',
          'linda',
          'magic',
          'merlin',
          'newyork',
          'soccer',
          'thomas',
          'wizard',
          'asdfgh',
          'bandit',
          'batman',
          'boris',
          'butthead',
          'dorothy',
          'eeyore',
          'fishing',
          'football',
          'george',
          'happy',
          'iloveyou',
          'jennifer',
          'jonathan',
          'love',
          'marina',
          'master',
          'missy',
          'monday',
          'monkey',
          'natasha',
          'ncc1701',
          'pamela',
          'pepper',
          'piglet',
          'poohbear',
          'pookie',
          'rabbit',
          'rachel',
          'rocket',
          'rose',
          'smile',
          'sparky',
          'spring',
          'steven',
          'success',
          'sunshine',
          'victoria',
          'whatever',
          'zapata',
          '8675309',
          'amanda',
          'andy',
          'angel',
          'august',
          'barney',
          'biteme',
          'boomer',
          'brian',
          'casey',
          'cowboy',
          'delta',
          'doctor',
          'fisher',
          'island',
          'john',
          'joshua',
          'karen',
          'marley',
          'orange',
          'please',
          'rascal',
          'richard',
          'sarah',
          'scooter',
          'shalom',
          'silver',
          'skippy',
          'stanley',
          'taylor',
          'welcome',
          'zephyr',
          '111111',
          'aaaaaa',
          'access',
          'albert',
          'alexander',
          'andrea',
          'anna',
          'anthony',
          'asdfjkl;',
          'ashley',
          'basketball',
          'beavis',
          'black',
          'bob',
          'booboo',
          'bradley',
          'brandon',
          'buddy',
          'caitlin',
          'camaro',
          'charlie',
          'chicken',
          'chris',
          'cindy',
          'cricket',
          'dakota',
          'dallas',
          'daniel',
          'david',
          'debbie',
          'dolphin',
          'elephant',
          'emily',
          'friend',
          'fucker',
          'ginger',
          'goodluck',
          'hammer',
          'heather',
          'iceman',
          'jason',
          'jessica',
          'jesus',
          'joseph',
          'jupiter',
          'justin',
          'kevin',
          'knight',
          'lacrosse',
          'lakers',
          'lizard',
          'madison',
          'mary',
          'mother',
          'muffin',
          'murphy',
          'nirvana',
          'paris',
          'pentium',
          'phoenix',
          'picture',
          'rainbow',
          'sandy',
          'saturn',
          'scott',
          'shannon',
          'shithead',
          'skeeter',
          'sophie',
          'special',
          'stephanie',
          'stephen',
          'steve',
          'sweetie',
          'teacher',
          'tennis',
          'test',
          'test123',
          'tommy',
          'topgun',
          'tristan',
          'wally',
          'william',
          'wilson',
          '1q2w3e',
          '654321',
          '666666',
          'a12345',
          'a1b2c3d4',
          'alpha',
          'amber',
          'angela',
          'angie',
          'archie',
          'asdf',
          'blazer',
          'bond007',
          'booger',
          'charles',
          'christin',
          'claire',
          'control',
          'danny',
          'david1',
          'dennis',
          'digital',
          'disney',
          'edward',
          'elvis',
          'felix',
          'flipper',
          'franklin',
          'frodo',
          'honda',
          'horses',
          'hunter',
          'indigo',
          'james',
          'jasper',
          'jeremy',
          'julian',
          'kelsey',
          'killer',
          'lauren',
          'marie',
          'maryjane',
          'matrix',
          'maverick',
          'mayday',
          'mercury',
          'mitchell',
          'morgan',
          'mountain',
          'niners',
          'nothing',
          'oliver',
          'peace',
          'peanut',
          'pearljam',
          'phantom',
          'popcorn',
          'princess',
          'psycho',
          'pumpkin',
          'purple',
          'randy',
          'rebecca',
          'reddog',
          'robert',
          'rocky',
          'roses',
          'salmon',
          'samson',
          'sharon',
          'sierra',
          'smokey',
          'startrek',
          'steelers',
          'stimpy',
          'sunflower',
          'superman',
          'support',
          'sydney',
          'techno',
          'walter',
          'willie',
          'willow',
          'winner',
          'ziggy',
          'zxcvbnm',
          'alaska',
          'alexis',
          'alice',
          'animal',
          'apples',
          'barbara',
          'benjamin',
          'billy',
          'blue',
          'bluebird',
          'bobby',
          'bonnie',
          'bubba',
          'camera',
          'chocolate',
          'clark',
          'claudia',
          'cocacola',
          'compton',
          'connect',
          'cookie',
          'cruise',
          'douglas',
          'dreamer',
          'dreams',
          'duckie',
          'eagles',
          'eddie',
          'einstein',
          'enter',
          'explorer',
          'faith',
          'family',
          'ferrari',
          'flamingo',
          'flower',
          'foxtrot',
          'francis',
          'freddy',
          'friday',
          'froggy',
          'giants',
          'gizmo',
          'global',
          'goofy',
          'happy1',
          'hendrix',
          'henry',
          'herman',
          'homer',
          'honey',
          'house',
          'houston',
          'iguana',
          'indiana',
          'insane',
          'inside',
          'irish',
          'ironman',
          'jake',
          'jasmin',
          'jeanne',
          'jerry',
          'joey',
          'justice',
          'katherine',
          'kermit',
          'kitty',
          'koala',
          'larry',
          'leslie',
          'logan',
          'lucky',
          'mark',
          'martin',
          'matt',
          'minnie',
          'misty',
          'mitch',
          'mouse',
          'nancy',
          'nascar',
          'nelson',
          'pantera',
          'parker',
          'penguin',
          'peter',
          'piano',
          'pizza',
          'prince',
          'punkin',
          'pyramid',
          'raymond',
          'robin',
          'roger',
          'rosebud',
          'route66',
          'royal',
          'running',
          'sadie',
          'sasha',
          'security',
          'sheena',
          'sheila',
          'skiing',
          'snapple',
          'snowball',
          'sparrow',
          'spencer',
          'spike',
          'star',
          'stealth',
          'student',
          'sunny',
          'sylvia',
          'tamara',
          'taurus',
          'teresa',
          'theresa',
          'thunderbird',
          'tigers',
          'tony',
          'toyota',
          'travel',
          'tuesday',
          'victory',
          'viper1',
          'wesley',
          'whisky',
          'winnie',
          'winter',
          'wolves',
          'xyz123',
          'zorro',
          '123123',
          '1234567',
          '696969',
          '888888',
          'Anthony',
          'Joshua',
          'Matthew',
          'Tigger',
          'aaron',
          'abby',
          'abcdef',
          'adidas',
          'adrian',
          'alfred',
          'arthur',
          'athena',
          'austin',
          'awesome',
          'badger',
          'bamboo',
          'beagle',
          'bears',
          'beatles',
          'beautiful',
          'beaver',
          'benny',
          'bigmac',
          'bingo',
          'bitch',
          'blonde',
          'boogie',
          'boston',
          'brenda',
          'bright',
          'bubba1',
          'bubbles',
          'buffy',
          'button',
          'buttons',
          'cactus',
          'candy',
          'captain',
          'carlos',
          'caroline',
          'carrie',
          'casper',
          'catch22',
          'chance',
          'charity',
          'charlotte',
          'cheese',
          'cheryl',
          'chloe',
          'chris1',
          'clancy',
          'compaq',
          'conrad',
          'cooper',
          'cooter',
          'copper',
          'cosmos',
          'cougar',
          'cracker',
          'crawford',
          'crystal',
          'curtis',
          'cyclone',
          'dance',
          'diablo',
          'dollars',
          'dookie',
          'dumbass',
          'dundee',
          'elizabeth',
          'eric',
          'europe',
          'farmer',
          'firebird',
          'fletcher',
          'fluffy',
          'france',
          'freak1',
          'friends',
          'fuckoff',
          'gabriel',
          'galaxy',
          'gambit',
          'garden',
          'garfield',
          'garnet',
          'genesis',
          'genius',
          'godzilla',
          'golfer',
          'goober',
          'grace',
          'greenday',
          'groovy',
          'grover',
          'guitar',
          'hacker',
          'harry',
          'hazel',
          'hector',
          'herbert',
          'horizon',
          'hornet',
          'howard',
          'icecream',
          'imagine',
          'impala',
          'jack',
          'janice',
          'jasmine',
          'jason1',
          'jeanette',
          'jeffrey',
          'jenifer',
          'jenni',
          'jesus1',
          'jewels',
          'joker',
          'julie',
          'julie1',
          'junior',
          'justin1',
          'kathleen',
          'keith',
          'kelly',
          'kelly1',
          'kennedy',
          'kevin1',
          'knicks',
          'larry1',
          'leonard',
          'lestat',
          'library',
          'lincoln',
          'lionking',
          'london',
          'louise',
          'lucky1',
          'lucy',
          'maddog',
          'margaret',
          'mariposa',
          'marlboro',
          'martin1',
          'marty',
          'master1',
          'mensuck',
          'mercedes',
          'metal',
          'midori',
          'mikey',
          'millie',
          'mirage',
          'molly',
          'monet',
          'money1',
          'monica',
          'monopoly',
          'mookie',
          'moose',
          'moroni',
          'music',
          'naomi',
          'nathan',
          'nguyen',
          'nicholas',
          'nicole',
          'nimrod',
          'october',
          'olive',
          'olivia',
          'online',
          'oscar',
          'oxford',
          'pacific',
          'painter',
          'peaches',
          'penelope',
          'pepsi',
          'petunia',
          'philip',
          'phoenix1',
          'photo',
          'pickle',
          'player',
          'poiuyt',
          'porsche',
          'porter',
          'puppy',
          'python',
          'quality',
          'raquel',
          'raven',
          'remember',
          'robbie',
          'robert1',
          'roman',
          'rugby',
          'runner',
          'russell',
          'ryan',
          'sailing',
          'sailor',
          'samantha',
          'savage',
          'scarlett',
          'school',
          'sean',
          'seven',
          'shadow1',
          'sheba',
          'shelby',
          'shit',
          'shoes',
          'simba',
          'simple',
          'skipper',
          'smiley',
          'snake',
          'snickers',
          'sniper',
          'snoopdog',
          'snowman',
          'sonic',
          'spitfire',
          'sprite',
          'spunky',
          'starwars',
          'station',
          'stella',
          'stingray',
          'storm',
          'stormy',
          'stupid',
          'sunny1',
          'sunrise',
          'surfer',
          'susan',
          'tammy',
          'tango',
          'tanya',
          'teddy1',
          'theboss',
          'theking',
          'thumper',
          'tina',
          'tintin',
          'tomcat',
          'trebor',
          'trevor',
          'tweety',
          'unicorn',
          'valentine',
          'valerie',
          'vanilla',
          'veronica',
          'victor',
          'vincent',
          'viper',
          'warrior',
          'warriors',
          'weasel',
          'wheels',
          'wilbur',
          'winston',
          'wisdom',
          'wombat',
          'xavier',
          'yellow',
          'zeppelin',
          '1111',
          '1212',
          'Andrew',
          'Family',
          'Friends',
          'Michael',
          'Michelle',
          'Snoopy',
          'abcd1234',
          'abcdefg',
          'abigail',
          'account',
          'adam',
          'alex1',
          'alice1',
          'allison',
          'alpine',
          'andre1',
          'andrea1',
          'angel1',
          'anita',
          'annette',
          'antares',
          'apache',
          'apollo',
          'aragorn',
          'arizona',
          'arnold',
          'arsenal',
          'asdfasdf',
          'asdfg',
          'asdfghjk',
          'avenger',
          'baby',
          'babydoll',
          'bailey',
          'banana',
          'barry',
          'basket',
          'batman1',
          'beaner',
          'beast',
          'beatrice',
          'bella',
          'bertha',
          'bigben',
          'bigdog',
          'biggles',
          'bigman',
          'binky',
          'biology',
          'bishop',
          'blondie',
          'bluefish',
          'bobcat',
          'bosco',
          'braves',
          'brazil',
          'bruce',
          'bruno',
          'brutus',
          'buffalo',
          'bulldog',
          'bullet',
          'bullshit',
          'bunny',
          'business',
          'butch',
          'butler',
          'butter',
          'california',
          'carebear',
          'carol',
          'carol1',
          'carole',
          'cassie',
          'castle',
          'catalina',
          'catherine',
          'cccccc',
          'celine',
          'center',
          'champion',
          'chanel',
          'chaos',
          'chelsea1',
          'chester1',
          'chicago',
          'chico',
          'christian',
          'christy',
          'church',
          'cinder',
          'colleen',
          'colorado',
          'columbia',
          'commander',
          'connie',
          'cookies',
          'cooking',
          'corona',
          'cowboys',
          'coyote',
          'craig',
          'creative',
          'cuddles',
          'cuervo',
          'cutie',
          'daddy',
          'daisy',
          'daniel1',
          'danielle',
          'davids',
          'death',
          'denis',
          'derek',
          'design',
          'destiny',
          'diana',
          'diane',
          'dickhead',
          'digger',
          'dodger',
          'donna',
          'dougie',
          'dragonfly',
          'dylan',
          'eagle',
          'eclipse',
          'electric',
          'emerald',
          'etoile',
          'excalibur',
          'express',
          'fender',
          'fiona',
          'fireman',
          'flash',
          'florida',
          'flowers',
          'foster',
          'francesco',
          'francine',
          'francois',
          'frank',
          'french',
          'fuckface',
          'gemini',
          'general',
          'gerald',
          'germany',
          'gilbert',
          'goaway',
          'golden',
          'goldfish',
          'goose',
          'gordon',
          'graham',
          'grant',
          'gregory',
          'gretchen',
          'gunner',
          'hannah',
          'harold',
          'harrison',
          'harvey',
          'hawkeye',
          'heaven',
          'heidi',
          'helen',
          'helena',
          'hithere',
          'hobbit',
          'ibanez',
          'idontknow',
          'integra',
          'ireland',
          'irene',
          'isaac',
          'isabel',
          'jackass',
          'jackie',
          'jackson',
          'jaguar',
          'jamaica',
          'japan',
          'jenny1',
          'jessie',
          'johan',
          'johnny',
          'joker1',
          'jordan23',
          'judith',
          'julia',
          'jumanji',
          'kangaroo',
          'karen1',
          'kathy',
          'keepout',
          'keith1',
          'kenneth',
          'kimberly',
          'kingdom',
          'kitkat',
          'kramer',
          'kristen',
          'laura',
          'laurie',
          'lawrence',
          'lawyer',
          'legend',
          'liberty',
          'light',
          'lindsay',
          'lindsey',
          'lisa',
          'liverpool',
          'lola',
          'lonely',
          'louis',
          'lovely',
          'loveme',
          'lucas',
          'madonna',
          'malcolm',
          'malibu',
          'marathon',
          'marcel',
          'maria1',
          'mariah',
          'mariah1',
          'marilyn',
          'mario',
          'marvin',
          'maurice',
          'maxine',
          'maxwell',
          'me',
          'meggie',
          'melanie',
          'melissa',
          'melody',
          'mexico',
          'michael1',
          'michele',
          'midnight',
          'mike1',
          'miracle',
          'misha',
          'mishka',
          'molly1',
          'monique',
          'montreal',
          'moocow',
          'moore',
          'morris',
          'mouse1',
          'mulder',
          'nautica',
          'nellie',
          'newton',
          'nick',
          'nirvana1',
          'nissan',
          'norman',
          'notebook',
          'ocean',
          'olivier',
          'ollie',
          'oranges',
          'oregon',
          'orion',
          'panda',
          'pandora',
          'panther',
          'passion',
          'patricia',
          'pearl',
          'peewee',
          'pencil',
          'penny',
          'people',
          'percy',
          'person',
          'peter1',
          'petey',
          'picasso',
          'pierre',
          'pinkfloyd',
          'polaris',
          'police',
          'pookie1',
          'poppy',
          'power',
          'predator',
          'preston',
          'q1w2e3',
          'queen',
          'queenie',
          'quentin',
          'ralph',
          'random',
          'rangers',
          'raptor',
          'reality',
          'redrum',
          'remote',
          'reynolds',
          'rhonda',
          'ricardo',
          'ricardo1',
          'ricky',
          'river',
          'roadrunner',
          'robinhood',
          'rocknroll',
          'rocky1',
          'ronald',
          'roxy',
          'ruthie',
          'sabrina',
          'sakura',
          'sally',
          'sampson',
          'samuel',
          'sandra',
          'santa',
          'sapphire',
          'scarlet',
          'scorpio',
          'scott1',
          'scottie',
          'scruffy',
          'seattle',
          'serena',
          'shanti',
          'shark',
          'shogun',
          'simon',
          'singer',
          'skull',
          'skywalker',
          'slacker',
          'smashing',
          'smiles',
          'snowflake',
          'snuffy',
          'soccer1',
          'soleil',
          'sonny',
          'spanky',
          'speedy',
          'spider',
          'spooky',
          'stacey',
          'star69',
          'start',
          'steven1',
          'stinky',
          'strawberry',
          'stuart',
          'sugar',
          'sundance',
          'superfly',
          'suzanne',
          'suzuki',
          'swimmer',
          'swimming',
          'system',
          'taffy',
          'tarzan',
          'teddy',
          'teddybear',
          'terry',
          'theatre',
          'thunder',
          'thursday',
          'tinker',
          'tootsie',
          'tornado',
          'tracy',
          'tricia',
          'trident',
          'trojan',
          'truman',
          'trumpet',
          'tucker',
          'turtle',
          'tyler',
          'utopia',
          'voyager',
          'warcraft',
          'warlock',
          'warren',
          'water',
          'wayne',
          'wendy',
          'williams',
          'willy',
          'winona',
          'woody',
          'woofwoof',
          'wrangler',
          'wright',
          'xfiles',
          'xxxxxx',
          'yankees',
          'yvonne',
          'zebra',
          'zenith',
          'zigzag',
          'zombie',
          'zxc123',
          'zxcvb',
          '000000',
          '007007',
          '11111',
          '11111111',
          '123321',
          '171717',
          '181818',
          '1a2b3c',
          '1chris',
          '4runner',
          '54321',
          '55555',
          '6969',
          '7777777',
          '789456',
          '88888888',
          'Alexis',
          'Bailey',
          'Charlie',
          'Chris',
          'Daniel',
          'Dragon',
          'Elizabeth',
          'HARLEY',
          'Heather',
          'Jennifer',
          'Jessica',
          'Jordan',
          'KILLER',
          'Nicholas',
          'Password',
          'Princess',
          'Purple',
          'Rebecca',
          'Robert',
          'Shadow',
          'Steven',
          'Summer',
          'Sunshine',
          'Superman',
          'Taylor',
          'Thomas',
          'Victoria',
          'abcd123',
          'abcde',
          'accord',
          'active',
          'africa',
          'airborne',
          'alfaro',
          'alicia',
          'aliens',
          'alina',
          'aline',
          'alison',
          'allen',
          'aloha',
          'alpha1',
          'althea',
          'altima',
          'amanda1',
          'amazing',
          'america',
          'amour',
          'anderson',
          'andre',
          'andrew1',
          'andromeda',
          'angels',
          'angie1',
          'annie',
          'anything',
          'apple1',
          'apple2',
          'applepie',
          'april',
          'aquarius',
          'ariane',
          'ariel',
          'arlene',
          'artemis',
          'asdf1234',
          'asdfjkl',
          'ashley1',
          'ashraf',
          'ashton',
          'asterix',
          'attila',
          'autumn',
          'avatar',
          'babes',
          'bambi',
          'barbie',
          'barney1',
          'barrett',
          'bball',
          'beaches',
          'beanie',
          'beans',
          'beauty',
          'becca',
          'belize',
          'belle',
          'belmont',
          'benji',
          'benson',
          'bernardo',
          'berry',
          'betsy',
          'betty',
          'bigboss',
          'bigred',
          'billy1',
          'birdie',
          'birthday',
          'biscuit',
          'bitter',
          'blackjack',
          'blah',
          'blanche',
          'blood',
          'blowjob',
          'blowme',
          'blueeyes',
          'blues',
          'bogart',
          'bombay',
          'boobie',
          'boots',
          'bootsie',
          'boxers',
          'brandi',
          'brent',
          'brewster',
          'bridge',
          'bronco',
          'bronte',
          'brooke',
          'brother',
          'bryan',
          'bubble',
          'buddha',
          'budgie',
          'burton',
          'butterfly',
          'byron',
          'calendar',
          'calvin1',
          'camel',
          'camille',
          'campbell',
          'camping',
          'cancer',
          'canela',
          'cannon',
          'carbon',
          'carnage',
          'carolyn',
          'carrot',
          'cascade',
          'catfish',
          'cathy',
          'catwoman',
          'cecile',
          'celica',
          'change',
          'chantal',
          'charger',
          'cherry',
          'chiara',
          'chiefs',
          'china',
          'chris123',
          'christ1',
          'christmas',
          'christopher',
          'chuck',
          'cindy1',
          'cinema',
          'civic',
          'claude',
          'clueless',
          'cobain',
          'cobra',
          'cody',
          'colette',
          'college',
          'colors',
          'colt45',
          'confused',
          'cool',
          'corvette',
          'cosmo',
          'country',
          'crusader',
          'cunningham',
          'cupcake',
          'cynthia',
          'dagger',
          'dammit',
          'dancer',
          'daphne',
          'darkstar',
          'darren',
          'darryl',
          'darwin',
          'deborah',
          'december',
          'deedee',
          'deeznuts',
          'delano',
          'delete',
          'demon',
          'denise',
          'denny',
          'desert',
          'deskjet',
          'detroit',
          'devil',
          'devine',
          'devon',
          'dexter',
          'dianne',
          'diesel',
          'director',
          'dixie',
          'dodgers',
          'doggy',
          'dollar',
          'dolly',
          'dominique',
          'domino',
          'dontknow',
          'doogie',
          'doudou',
          'downtown',
          'dragon1',
          'driver',
          'dude',
          'dudley',
          'dutchess',
          'dwight',
          'eagle1',
          'easter',
          'eastern',
          'edith',
          'edmund',
          'eight',
          'element',
          'elissa',
          'ellen',
          'elliot',
          'empire',
          'enigma',
          'enterprise',
          'erin',
          'escort',
          'estelle',
          'eugene',
          'evelyn',
          'explore',
          'family1',
          'fatboy',
          'felipe',
          'ferguson',
          'ferret',
          'ferris',
          'fireball',
          'fishes',
          'fishie',
          'flight',
          'florida1',
          'flowerpot',
          'forward',
          'freddie',
          'freebird',
          'freeman',
          'frisco',
          'fritz',
          'froggie',
          'froggies',
          'frogs',
          'fucku',
          'future',
          'gabby',
          'games',
          'garcia',
          'gaston',
          'gateway',
          'george1',
          'georgia',
          'german',
          'germany1',
          'getout',
          'ghost',
          'gibson',
          'giselle',
          'gmoney',
          'goblin',
          'goblue',
          'gollum',
          'grandma',
          'gremlin',
          'grizzly',
          'grumpy',
          'guess',
          'guitar1',
          'gustavo',
          'haggis',
          'haha',
          'hailey',
          'halloween',
          'hamilton',
          'hamlet',
          'hanna',
          'hanson',
          'happy123',
          'happyday',
          'hardcore',
          'harley1',
          'harriet',
          'harris',
          'harvard',
          'health',
          'heart',
          'heather1',
          'heather2',
          'hedgehog',
          'helene',
          'hello1',
          'hello123',
          'hellohello',
          'hermes',
          'heythere',
          'highland',
          'hilda',
          'hillary',
          'history',
          'hitler',
          'hobbes',
          'holiday',
          'holly',
          'honda1',
          'hongkong',
          'hootie',
          'horse',
          'hotrod',
          'hudson',
          'hummer',
          'huskies',
          'idiot',
          'iforget',
          'iloveu',
          'impact',
          'indonesia',
          'irina',
          'isabelle',
          'israel',
          'italia',
          'italy',
          'jackie1',
          'jacob',
          'jakey',
          'james1',
          'jamesbond',
          'jamie',
          'jamjam',
          'jeffrey1',
          'jennie',
          'jenny',
          'jensen',
          'jesse',
          'jesse1',
          'jester',
          'jethro',
          'jimbob',
          'jimmy',
          'joanna',
          'joelle',
          'john316',
          'jordie',
          'jorge',
          'josh',
          'journey',
          'joyce',
          'jubilee',
          'jules',
          'julien',
          'juliet',
          'junebug',
          'juniper',
          'justdoit',
          'karin',
          'karine',
          'karma',
          'katerina',
          'katie',
          'katie1',
          'kayla',
          'keeper',
          'keller',
          'kendall',
          'kenny',
          'ketchup',
          'kings',
          'kissme',
          'kitten',
          'kittycat',
          'kkkkkk',
          'kristi',
          'kristine',
          'labtec',
          'laddie',
          'ladybug',
          'lance',
          'laurel',
          'lawson',
          'leader',
          'leland',
          'lemon',
          'lester',
          'letter',
          'letters',
          'lexus1',
          'libra',
          'lights',
          'lionel',
          'little',
          'lizzy',
          'lolita',
          'lonestar',
          'longhorn',
          'looney',
          'loren',
          'lorna',
          'loser',
          'lovers',
          'loveyou',
          'lucia',
          'lucifer',
          'lucky14',
          'maddie',
          'madmax',
          'magic1',
          'magnum',
          'maiden',
          'maine',
          'management',
          'manson',
          'manuel',
          'marcus',
          'maria',
          'marielle',
          'marine',
          'marino',
          'marshall',
          'martha',
          'maxmax',
          'meatloaf',
          'medical',
          'megan',
          'melina',
          'memphis',
          'mermaid',
          'miami',
          'michel',
          'michigan',
          'mickey1',
          'microsoft',
          'mikael',
          'milano',
          'miles',
          'millenium',
          'million',
          'miranda',
          'miriam',
          'mission',
          'mmmmmm',
          'mobile',
          'monkey1',
          'monroe',
          'montana',
          'monty',
          'moomoo',
          'moonbeam',
          'morpheus',
          'motorola',
          'movies',
          'mozart',
          'munchkin',
          'murray',
          'mustang1',
          'nadia',
          'nadine',
          'napoleon',
          'nation',
          'national',
          'nestle',
          'newlife',
          'newyork1',
          'nichole',
          'nikita',
          'nikki',
          'nintendo',
          'nokia',
          'nomore',
          'normal',
          'norton',
          'noway',
          'nugget',
          'number9',
          'numbers',
          'nurse',
          'nutmeg',
          'ohshit',
          'oicu812',
          'omega',
          'openup',
          'orchid',
          'oreo',
          'orlando',
          'packard',
          'packers',
          'paloma',
          'pancake',
          'panic',
          'parola',
          'parrot',
          'partner',
          'pascal',
          'patches',
          'patriots',
          'paula',
          'pauline',
          'payton',
          'peach',
          'peanuts',
          'pedro1',
          'peggy',
          'perfect',
          'perry',
          'peterpan',
          'philips',
          'phillips',
          'phone',
          'pierce',
          'pigeon',
          'pink',
          'pioneer',
          'piper1',
          'pirate',
          'pisces',
          'playboy',
          'pluto',
          'poetry',
          'pontiac',
          'pookey',
          'popeye',
          'prayer',
          'precious',
          'prelude',
          'premier',
          'puddin',
          'pulsar',
          'pussy',
          'pussy1',
          'qwert',
          'qwerty12',
          'qwertyui',
          'rabbit1',
          'rachelle',
          'racoon',
          'rambo',
          'randy1',
          'ravens',
          'redman',
          'redskins',
          'reggae',
          'reggie',
          'renee',
          'renegade',
          'rescue',
          'revolution',
          'richard1',
          'richards',
          'richmond',
          'riley',
          'ripper',
          'robby',
          'roberts',
          'rock',
          'rocket1',
          'rockie',
          'rockon',
          'roger1',
          'rogers',
          'roland',
          'rommel',
          'rookie',
          'rootbeer',
          'rosie',
          'rufus',
          'rusty',
          'ruthless',
          'sabbath',
          'sabina',
          'safety',
          'saint',
          'samiam',
          'sammie',
          'sammy',
          'samsam',
          'sandi',
          'sanjose',
          'saphire',
          'sarah1',
          'saskia',
          'sassy',
          'saturday',
          'science',
          'scooby',
          'scoobydoo',
          'scooter1',
          'scorpion',
          'scotty',
          'scouts',
          'search',
          'september',
          'server',
          'seven7',
          'sexy',
          'shaggy',
          'shanny',
          'shaolin',
          'shasta',
          'shayne',
          'shelly',
          'sherry',
          'shirley',
          'shorty',
          'shotgun',
          'sidney',
          'simba1',
          'sinatra',
          'sirius',
          'skate',
          'skipper1',
          'skyler',
          'slayer',
          'sleepy',
          'slider',
          'smile1',
          'smitty',
          'smoke',
          'snakes',
          'snapper',
          'snoop',
          'solomon',
          'sophia',
          'space',
          'sparks',
          'spartan',
          'spike1',
          'sponge',
          'spurs',
          'squash',
          'stargate',
          'starlight',
          'stars',
          'steph1',
          'steve1',
          'stevens',
          'stewart',
          'stone',
          'stranger',
          'stretch',
          'strong',
          'studio',
          'stumpy',
          'sucker',
          'suckme',
          'sultan',
          'summit',
          'sunfire',
          'sunset',
          'super',
          'superstar',
          'surfing',
          'susan1',
          'sutton',
          'sweden',
          'sweetpea',
          'sweety',
          'swordfish',
          'tabatha',
          'tacobell',
          'taiwan',
          'tamtam',
          'tanner',
          'target',
          'tasha',
          'tattoo',
          'tequila',
          'terry1',
          'texas',
          'thankyou',
          'theend',
          'thompson',
          'thrasher',
          'tiger2',
          'timber',
          'timothy',
          'tinkerbell',
          'topcat',
          'topher',
          'toshiba',
          'tototo',
          'travis',
          'treasure',
          'trees',
          'tricky',
          'trish',
          'triton',
          'trombone',
          'trouble',
          'trucker',
          'turbo',
          'twins',
          'tyler1',
          'ultimate',
          'unique',
          'united',
          'ursula',
          'vacation',
          'valley',
          'vampire',
          'vanessa',
          'venice',
          'venus',
          'vermont',
          'vicki',
          'vicky',
          'victor1',
          'vincent1',
          'violet',
          'violin',
          'virgil',
          'virginia',
          'vision',
          'volley',
          'voodoo',
          'vortex',
          'waiting',
          'wanker',
          'warner',
          'water1',
          'wayne1',
          'webster',
          'weezer',
          'wendy1',
          'western',
          'white',
          'whitney',
          'whocares',
          'wildcat',
          'william1',
          'wilma',
          'window',
          'winniethepooh',
          'wolfgang',
          'wolverine',
          'wonder',
          'xxxxxxxx',
          'yamaha',
          'yankee',
          'yogibear',
          'yolanda',
          'yomama',
          'yvette',
          'zachary',
          'zebras',
          'zxcvbn',
          '00000000',
          '121212',
          '1234qwer',
          '131313',
          '13579',
          '90210',
          '99999999',
          'ABC123',
          'action',
          'amelie',
          'anaconda',
          'apollo13',
          'artist',
          'asshole',
          'benoit',
          'bernard',
          'bernie',
          'bigbird',
          'blizzard',
          'bluesky',
          'bonjour',
          'caesar',
          'cardinal',
          'carolina',
          'cesar',
          'chandler',
          'chapman',
          'charlie1',
          'chevy',
          'chiquita',
          'chocolat',
          'coco',
          'cougars',
          'courtney',
          'dolphins',
          'dominic',
          'donkey',
          'dusty',
          'eminem',
          'energy',
          'fearless',
          'forest',
          'forever',
          'glenn',
          'guinness',
          'hotdog',
          'indian',
          'jared',
          'jimbo',
          'johnson',
          'jojo',
          'josie',
          'kristin',
          'lloyd',
          'lorraine',
          'lynn',
          'maxime',
          'memory',
          'mimi',
          'mirror',
          'nebraska',
          'nemesis',
          'network',
          'nigel',
          'oatmeal',
          'patton',
          'pedro',
          'planet',
          'players',
          'portland',
          'praise',
          'psalms',
          'qwaszx',
          'raiders',
          'rambo1',
          'rancid',
          'shawn',
          'shelley',
          'softball',
          'speedo',
          'sports',
          'ssssss',
          'steele',
          'steph',
          'stephani',
          'sunday',
          'tiffany',
          'tigre',
          'toronto',
          'trixie',
          'undead',
          'valentin',
          'velvet',
          'viking',
          'walker',
          'watson',
          'young',
          'babygirl',
          'pretty',
          'hottie',
          'teamo',
          '987654321',
          'naruto',
          'spongebob',
          'daniela',
          'princesa',
          'christ',
          'blessed',
          'single',
          'qazwsx',
          'pokemon',
          'iloveyou1',
          'iloveyou2',
          'fuckyou1',
          'hahaha',
          'poop',
          'blessing',
          'blahblah',
          'blink182',
          '123qwe',
          'trinity',
          'passw0rd',
          'google',
          'looking',
          'spirit',
          'iloveyou!',
          'qwerty1',
          'onelove',
          'mylove',
          '222222',
          'ilovegod',
          'football1',
          'loving',
          'emmanuel',
          '1q2w3e4r',
          'red123',
          'blabla',
          '112233',
          'hallo',
          'spiderman',
          'simpsons',
          'monster',
          'november',
          'brooklyn',
          'poopoo',
          'darkness',
          '159753',
          'pineapple',
          'chester',
          '1qaz2wsx',
          'drowssap',
          'monkey12',
          'wordpass',
          'q1w2e3r4',
          'coolness',
          '11235813',
          'something',
          'alexandra',
          'estrella',
          'miguel',
          'iloveme',
          'sayang',
          'princess1',
          '555555',
          '999999',
          'alejandro',
          'brittany',
          'alejandra',
          'tequiero',
          'antonio',
          '987654',
          '00000',
          'fernando',
          'corazon',
          'cristina',
          'kisses',
          'myspace',
          'rebelde',
          'babygurl',
          'alyssa',
          'mahalkita',
          'gabriela',
          'pictures',
          'hellokitty',
          'babygirl1',
          'angelica',
          'mahalko',
          'mariana',
          'eduardo',
          'andres',
          'ronaldo',
          'inuyasha',
          'adriana',
          'celtic',
          'samsung',
          'angelo',
          '456789',
          'sebastian',
          'karina',
          'hotmail',
          '0123456789',
          'barcelona',
          'cameron',
          'slipknot',
          'cutiepie',
          '50cent',
          'bonita',
          'maganda',
          'babyboy',
          'natalie',
          'cuteako',
          'javier',
          '789456123',
          '123654',
          'bowwow',
          'portugal',
          '777777',
          'volleyball',
          'january',
          'cristian',
          'bianca',
          'chrisbrown',
          '101010',
          'sweet',
          'panget',
          'benfica',
          'love123',
          'lollipop',
          'camila',
          'qwertyuiop',
          'harrypotter',
          'ihateyou',
          'christine',
          'lorena',
          'andreea',
          'charmed',
          'rafael',
          'brianna',
          'aaliyah',
          'johncena',
          'lovelove',
          'gangsta',
          '333333',
          'hiphop',
          'mybaby',
          'sergio',
          'metallica',
          'myspace1',
          'babyblue',
          'badboy',
          'fernanda',
          'westlife',
          'sasuke',
          'steaua',
          'roberto',
          'slideshow',
          'asdfghjkl',
          'santiago',
          'jayson',
          '5201314',
          'jerome',
          'gandako',
          'gatita',
          'babyko',
          '246810',
          'sweetheart',
          'chivas',
          'alberto',
          'valeria',
          'nicole1',
          '12345678910',
          'leonardo',
          'jayjay',
          'liliana',
          'sexygirl',
          '232323',
          'amores',
          'anthony1',
          'bitch1',
          'fatima',
          'miamor',
          'lover',
          'lalala',
          '252525',
          'skittles',
          'colombia',
          '159357',
          'manutd',
          '123456a',
          'britney',
          'katrina',
          'christina',
          'pasaway',
          'mahal',
          'tatiana',
          'cantik',
          '0123456',
          'teiubesc',
          '147258369',
          'natalia',
          'francisco',
          'amorcito',
          'paola',
          'angelito',
          'manchester',
          'mommy1',
          '147258',
          'amigos',
          'marlon',
          'linkinpark',
          '147852',
          'diego',
          '444444',
          'iverson',
          'andrei',
          'justine',
          'frankie',
          'pimpin',
          'fashion',
          'bestfriend',
          'england',
          'hermosa',
          '456123',
          '102030',
          'sporting',
          'hearts',
          'potter',
          'iloveu2',
          'number1',
          '212121',
          'truelove',
          'jayden',
          'savannah',
          'hottie1',
          'ganda',
          'scotland',
          'ilovehim',
          'shakira',
          'estrellita',
          'brandon1',
          'sweets',
          'familia',
          'love12',
          'omarion',
          'monkeys',
          'loverboy',
          'elijah',
          'ronnie',
          'mamita',
          '999999999',
          'broken',
          'rodrigo',
          'westside',
          'mauricio',
          'amigas',
          'preciosa',
          'shopping',
          'flores',
          'isabella',
          'martinez',
          'elaine',
          'friendster',
          'cheche',
          'gracie',
          'connor',
          'valentina',
          'darling',
          'santos',
          'joanne',
          'fuckyou2',
          'pebbles',
          'sunshine1',
          'gangster',
          'gloria',
          'darkangel',
          'bettyboop',
          'jessica1',
          'cheyenne',
          'dustin',
          'iubire',
          'a123456',
          'purple1',
          'bestfriends',
          'inlove',
          'batista',
          'karla',
          'chacha',
          'marian',
          'sexyme',
          'pogiako',
          'jordan1',
          '010203',
          'daddy1',
          'daddysgirl',
          'billabong',
          'pinky',
          'erika',
          'skater',
          'nenita',
          'tigger1',
          'gatito',
          'lokita',
          'maldita',
          'buttercup',
          'bambam',
          'glitter',
          '123789',
          'sister',
          'zacefron',
          'tokiohotel',
          'loveya',
          'lovebug',
          'bubblegum',
          'marissa',
          'cecilia',
          'lollypop',
          'nicolas',
          'puppies',
          'ariana',
          'chubby',
          'sexybitch',
          'roxana',
          'mememe',
          'susana',
          'baller',
          'hotstuff',
          'carter',
          'babylove',
          'angelina',
          'playgirl',
          'sweet16',
          '012345',
          'bhebhe',
          'marcos',
          'loveme1',
          'milagros',
          'lilmama',
          'beyonce',
          'lovely1',
          'catdog',
          'armando',
          'margarita',
          '151515',
          'loves',
          '202020',
          'gerard',
          'undertaker',
          'amistad',
          'capricorn',
          'delfin',
          'cheerleader',
          'password2',
          'PASSWORD',
          'lizzie',
          'matthew1',
          'enrique',
          'badgirl',
          '141414',
          'dancing',
          'cuteme',
          'amelia',
          'skyline',
          'angeles',
          'janine',
          'carlitos',
          'justme',
          'legolas',
          'michelle1',
          'cinderella',
          'jesuschrist',
          'ilovejesus',
          'tazmania',
          'tekiero',
          'thebest',
          'princesita',
          'lucky7',
          'jesucristo',
          'buddy1',
          'regina',
          'myself',
          'lipgloss',
          'jazmin',
          'rosita',
          'chichi',
          'pangit',
          'mierda',
          '741852963',
          'hernandez',
          'arturo',
          'silvia',
          'melvin',
          'celeste',
          'pussycat',
          'gorgeous',
          'honeyko',
          'mylife',
          'babyboo',
          'loveu',
          'lupita',
          'panthers',
          'hollywood',
          'alfredo',
          'musica',
          'hawaii',
          'sparkle',
          'kristina',
          'sexymama',
          'crazy',
          'scarface',
          '098765',
          'hayden',
          'micheal',
          '242424',
          '0987654321',
          'marisol',
          'jeremiah',
          'mhine',
          'isaiah',
          'lolipop',
          'butterfly1',
          'xbox360',
          'madalina',
          'anamaria',
          'yourmom',
          'jasmine1',
          'bubbles1',
          'beatriz',
          'diamonds',
          'friendship',
          'sweetness',
          'desiree',
          '741852',
          'hannah1',
          'bananas',
          'julius',
          'leanne',
          'marie1',
          'lover1',
          'twinkle',
          'february',
          'bebita',
          '87654321',
          'twilight',
          'imissyou',
          'pollito',
          'ashlee',
          'cookie1',
          '147852369',
          'beckham',
          'simone',
          'nursing',
          'torres',
          'damian',
          '123123123',
          'joshua1',
          'babyface',
          'dinamo',
          'mommy',
          'juliana',
          'cassandra',
          'redsox',
          'gundam',
          '0000',
          'ou812',
          'dave',
          'golf',
          'molson',
          'Monday',
          'newpass',
          'thx1138',
          '1',
          'Internet',
          'coke',
          'foobar',
          'abc',
          'fish',
          'fred',
          'help',
          'ncc1701d',
          'newuser',
          'none',
          'pat',
          'dog',
          'duck',
          'duke',
          'floyd',
          'guest',
          'joe',
          'kingfish',
          'micro',
          'sam',
          'telecom',
          'test1',
          '7777',
          'absolut',
          'babylon5',
          'backup',
          'bill',
          'bird33',
          'deliver',
          'fire',
          'flip',
          'galileo',
          'gopher',
          'hansolo',
          'jane',
          'jim',
          'mom',
          'passwd',
          'phil',
          'phish',
          'porsche911',
          'rain',
          'red',
          'sergei',
          'training',
          'truck',
          'video',
          'volvo',
          '007',
          '1969',
          '5683',
          'Bond007',
          'Friday',
          'Hendrix',
          'October',
          'Taurus',
          'aaa',
          'alexandr',
          'catalog',
          'challenge',
          'clipper',
          'coltrane',
          'cyrano',
          'dan',
          'dawn',
          'dean',
          'deutsch',
          'dilbert',
          'e-mail',
          'export',
          'ford',
          'fountain',
          'fox',
          'frog',
          'gabriell',
          'garlic',
          'goforit',
          'grateful',
          'hoops',
          'lady',
          'ledzep',
          'lee',
          'mailman',
          'mantra',
          'market',
          'mazda1',
          'metallic',
          'ncc1701e',
          'nesbitt',
          'open',
          'pete',
          'quest',
          'republic',
          'research',
          'supra',
          'tara',
          'testing',
          'xanadu',
          'xxxx',
          'zaphod',
          'zeus',
          '0007',
          '1022',
          '10sne1',
          '1973',
          '1978',
          '2000',
          '2222',
          '3bears',
          'Broadway',
          'Fisher',
          'Jeanne',
          'Killer',
          'Knight',
          'Master',
          'Pepper',
          'Sierra',
          'Tennis',
          'abacab',
          'abcd',
          'ace',
          'acropolis',
          'amy',
          'anders',
          'avenir',
          'basil',
          'bass',
          'beer',
          'ben',
          'bliss',
          'blowfish',
          'boss',
          'bridges',
          'buck',
          'bugsy',
          'bull',
          'cannondale',
          'canon',
          'catnip',
          'chip',
          'civil',
          'content',
          'cook',
          'cordelia',
          'crack1',
          'cyber',
          'daisie',
          'dark1',
          'database',
          'deadhead',
          'denali',
          'depeche',
          'dickens',
          'emmitt',
          'entropy',
          'farout',
          'farside',
          'feedback',
          'fidel',
          'firenze',
          'fish1',
          'fletch',
          'fool',
          'fozzie',
          'fun',
          'gargoyle',
          'gasman',
          'gold',
          'graphic',
          'hell',
          'image',
          'intern',
          'intrepid',
          'jeff',
          'jkl123',
          'joel',
          'johanna1',
          'kidder',
          'kim',
          'king',
          'kirk',
          'kris',
          'lambda',
          'leon',
          'logical',
          'lorrie',
          'major',
          'mariner',
          'mark1',
          'max',
          'media',
          'merlot',
          'midway',
          'mine',
          'mmouse',
          'moon',
          'mopar',
          'mortimer',
          'nermal',
          'nina',
          'olsen',
          'opera',
          'overkill',
          'pacers',
          'packer',
          'picard',
          'polar',
          'polo',
          'primus',
          'prometheus',
          'public',
          'radio',
          'rastafarian',
          'reptile',
          'rob',
          'robotech',
          'rodeo',
          'rolex',
          'rouge',
          'roy',
          'ruby',
          'salasana',
          'scarecrow',
          'scout',
          'scuba1',
          'sergey',
          'skibum',
          'skunk',
          'sound',
          'starter',
          'sting1',
          'sunbird',
          'tbird',
          'teflon',
          'temporal',
          'terminal',
          'the',
          'thejudge',
          'time',
          'toby',
          'today',
          'tokyo',
          'tree',
          'trout',
          'vader',
          'val',
          'valhalla',
          'windsurf',
          'wolf',
          'wolf1',
          'xcountry',
          'yoda',
          'yukon',
          '1213',
          '1214',
          '1225',
          '1313',
          '1818',
          '1975',
          '1977',
          '1991',
          '1kitty',
          '2001',
          '2020',
          '2112',
          '2kids',
          '333',
          '4444',
          '5050',
          '57chevy',
          '7dwarfs',
          'Animals',
          'Ariel',
          'Bismillah',
          'Booboo',
          'Boston',
          'Carol',
          'Computer',
          'Creative',
          'Curtis',
          'Denise',
          'Eagles',
          'Esther',
          'Fishing',
          'Freddy',
          'Gandalf',
          'Golden',
          'Goober',
          'Hacker',
          'Harley',
          'Henry',
          'Hershey',
          'Jackson',
          'Jersey',
          'Joanna',
          'Johnson',
          'Katie',
          'Kitten',
          'Liberty',
          'Lindsay',
          'Lizard',
          'Madeline',
          'Margaret',
          'Maxwell',
          'Money',
          'Monster',
          'Pamela',
          'Peaches',
          'Peter',
          'Phoenix',
          'Piglet',
          'Pookie',
          'Rabbit',
          'Raiders',
          'Random',
          'Russell',
          'Sammy',
          'Saturn',
          'Skeeter',
          'Smokey',
          'Sparky',
          'Speedy',
          'Sterling',
          'Theresa',
          'Thunder',
          'Vincent',
          'Willow',
          'Winnie',
          'Wolverine',
          'aaaa',
          'aardvark',
          'abbott',
          'acura',
          'admin',
          'admin1',
          'adrock',
          'aerobics',
          'agent',
          'airwolf',
          'ali',
          'alien',
          'allegro',
          'allstate',
          'altamira',
          'altima1',
          'andrew!',
          'ann',
          'anne',
          'anneli',
          'aptiva',
          'arrow',
          'asdf;lkj',
          'assmunch',
          'baraka',
          'barnyard',
          'bart',
          'bartman',
          'beasty',
          'beavis1',
          'bebe',
          'belgium',
          'beowulf',
          'beryl',
          'best',
          'bharat',
          'bichon',
          'bigal',
          'biker',
          'bilbo',
          'bills',
          'bimmer',
          'biochem',
          'birdy',
          'blinds',
          'blitz',
          'bluejean',
          'bogey',
          'bogus',
          'boulder',
          'bourbon',
          'boxer',
          'brain',
          'branch',
          'britain',
          'broker',
          'bucks',
          'buffett',
          'bugs',
          'bulls',
          'burns',
          'buzz',
          'c00per',
          'calgary',
          'camay',
          'carl',
          'cat',
          'cement',
          'cessna',
          'chad',
          'chainsaw',
          'chameleon',
          'chang',
          'chess',
          'chinook',
          'chouette',
          'chronos',
          'cicero',
          'circuit',
          'cirque',
          'cirrus',
          'clapton',
          'clarkson',
          'class',
          'claudel',
          'cleo',
          'cliff',
          'clock',
          'color',
          'comet',
          'concept',
          'concorde',
          'coolbean',
          'corky',
          'cornflake',
          'corwin',
          'cows',
          'crescent',
          'cross',
          'crowley',
          'cthulhu',
          'cunt',
          'current',
          'cutlass',
          'daedalus',
          'dagger1',
          'daily',
          'dale',
          'dana',
          'daytek',
          'dead',
          'decker',
          'dharma',
          'dillweed',
          'dipper',
          'disco',
          'dixon',
          'doitnow',
          'doors',
          'dork',
          'doug',
          'dutch',
          'effie',
          'ella',
          'elsie',
          'engage',
          'eric1',
          'ernie1',
          'escort1',
          'excel',
          'faculty',
          'fairview',
          'faust',
          'fenris',
          'finance',
          'first',
          'fishhead',
          'flanders',
          'fleurs',
          'flute',
          'flyboy',
          'flyer',
          'franka',
          'frederic',
          'free',
          'front242',
          'frontier',
          'fugazi',
          'funtime',
          'gaby',
          'gaelic',
          'gambler',
          'gammaphi',
          'garfunkel',
          'garth',
          'gary',
          'gateway2',
          'gator1',
          'gibbons',
          'gigi',
          'gilgamesh',
          'goat',
          'godiva',
          'goethe',
          'gofish',
          'good',
          'gramps',
          'gravis',
          'gray',
          'greed',
          'greg',
          'greg1',
          'greta',
          'gretzky',
          'guido',
          'gumby',
          'h2opolo',
          'hamid',
          'hank',
          'hawkeye1',
          'health1',
          'hello8',
          'help123',
          'helper',
          'homerj',
          'hoosier',
          'hope',
          'huang',
          'hugo',
          'hydrogen',
          'ib6ub9',
          'insight',
          'instructor',
          'integral',
          'iomega',
          'iris',
          'izzy',
          'jazz',
          'jean',
          'jeepster',
          'jetta1',
          'joanie',
          'josee',
          'joy',
          'julia2',
          'jumbo',
          'jump',
          'justice4',
          'kalamazoo',
          'kali',
          'kat',
          'kate',
          'kerala',
          'kids',
          'kiwi',
          'kleenex',
          'kombat',
          'lamer',
          'laser',
          'laserjet',
          'lassie1',
          'leblanc',
          'legal',
          'leo',
          'life',
          'lions',
          'liz',
          'logger',
          'logos',
          'loislane',
          'loki',
          'longer',
          'lori',
          'lost',
          'lotus',
          'lou',
          'macha',
          'macross',
          'madoka',
          'makeitso',
          'mallard',
          'marc',
          'math',
          'mattingly',
          'mechanic',
          'meister',
          'mercer',
          'merde',
          'merrill',
          'michal',
          'michou',
          'mickel',
          'minou',
          'mobydick',
          'modem',
          'mojo',
          'montana3',
          'montrose',
          'motor',
          'mowgli',
          'mulder1',
          'muscle',
          'neil',
          'neutrino',
          'newaccount',
          'nicklaus',
          'nightshade',
          'nightwing',
          'nike',
          'none1',
          'nopass',
          'nouveau',
          'novell',
          'oaxaca',
          'obiwan',
          'obsession',
          'orville',
          'otter',
          'ozzy',
          'packrat',
          'paint',
          'papa',
          'paradigm',
          'pass',
          'pavel',
          'peterk',
          'phialpha',
          'phishy',
          'piano1',
          'pianoman',
          'pianos',
          'pipeline',
          'plato',
          'play',
          'poetic',
          'print',
          'printing',
          'provider',
          'qqq111',
          'quebec',
          'qwer',
          'racer',
          'racerx',
          'radar',
          'rafiki',
          'raleigh',
          'rasta1',
          'redcloud',
          'redfish',
          'redwing',
          'redwood',
          'reed',
          'rene',
          'reznor',
          'rhino',
          'ripple',
          'rita',
          'robocop',
          'robotics',
          'roche',
          'roni',
          'rossignol',
          'rugger',
          'safety1',
          'saigon',
          'satori',
          'saturn5',
          'schnapps',
          'scotch',
          'scuba',
          'secret3',
          'seeker',
          'services',
          'sex',
          'shanghai',
          'shazam',
          'shelter',
          'sigmachi',
          'signal',
          'signature',
          'simsim',
          'skydive',
          'slick',
          'smegma',
          'smiths',
          'smurfy',
          'snow',
          'sober1',
          'sonics',
          'sony',
          'spazz',
          'sphynx',
          'spock',
          'spoon',
          'spot',
          'sprocket',
          'starbuck',
          'steel',
          'stephi',
          'sting',
          'stocks',
          'storage',
          'strat',
          'strato',
          'stud',
          'student2',
          'susanna',
          'swanson',
          'swim',
          'switzer',
          'system5',
          't-bone',
          'talon',
          'tarheel',
          'tata',
          'tazdevil',
          'tester',
          'testtest',
          'thisisit',
          'thorne',
          'tightend',
          'tim',
          'tom',
          'tool',
          'total',
          'toucan',
          'transfer',
          'transit',
          'transport',
          'trapper',
          'trash',
          'trophy',
          'tucson',
          'turbo2',
          'unity',
          'upsilon',
          'vedder',
          'vette',
          'vikram',
          'virago',
          'visual',
          'volcano',
          'walden',
          'waldo',
          'walleye',
          'webmaster',
          'wedge',
          'whale1',
          'whit',
          'whoville',
          'wibble',
          'will',
          'wombat1',
          'word',
          'world',
          'x-files',
          'xxx123',
          'zack',
          'zepplin',
          'zoltan',
          'zoomer',
          '123go',
          '21122112',
          '5555',
          '911',
          'FuckYou',
          'Fuckyou',
          'Gizmo',
          'Hello',
          'Michel',
          'Qwerty',
          'Windows',
          'angus',
          'aspen',
          'ass',
          'bird',
          'booster',
          'byteme',
          'cats',
          'changeit',
          'christia',
          'christoph',
          'classroom',
          'cloclo',
          'corrado',
          'dasha',
          'fiction',
          'french1',
          'fubar',
          'gator',
          'gilles',
          'gocougs',
          'hilbert',
          'hola',
          'home',
          'judy',
          'koko',
          'lulu',
          'mac',
          'macintosh',
          'mailer',
          'mars',
          'meow',
          'ne1469',
          'niki',
          'paul',
          'politics',
          'pomme',
          'property',
          'ruth',
          'sales',
          'salut',
          'scrooge',
          'skidoo',
          'spain',
          'surf',
          'sylvie',
          'symbol',
          'forum',
          'rotimi',
          'god',
          'saved',
          '2580',
          '1998',
          'xxx',
          '1928',
          '777',
          'info',
          'a',
          'netware',
          'sun',
          'tech',
          'doom',
          'mmm',
          'one',
          'ppp',
          '1911',
          '1948',
          '1996',
          '5252',
          'Champs',
          'Tuesday',
          'bach',
          'crow',
          'don',
          'draft',
          'hal9000',
          'herzog',
          'huey',
          'jethrotull',
          'jussi',
          'mail',
          'miki',
          'nicarao',
          'snowski',
          '1316',
          '1412',
          '1430',
          '1952',
          '1953',
          '1955',
          '1956',
          '1960',
          '1964',
          '1qw23e',
          '22',
          '2200',
          '2252',
          '3010',
          '3112',
          '4788',
          '6262',
          'Alpha',
          'Bastard',
          'Beavis',
          'Cardinal',
          'Celtics',
          'Cougar',
          'Darkman',
          'Figaro',
          'Fortune',
          'Geronimo',
          'Hammer',
          'Homer',
          'Janet',
          'Mellon',
          'Merlot',
          'Metallic',
          'Montreal',
          'Newton',
          'Paladin',
          'Peanuts',
          'Service',
          'Vernon',
          'Waterloo',
          'Webster',
          'aki123',
          'aqua',
          'aylmer',
          'beta',
          'bozo',
          'car',
          'chat',
          'chinacat',
          'cora',
          'courier',
          'dogbert',
          'eieio',
          'elina1',
          'fly',
          'funguy',
          'fuzz',
          'ggeorge',
          'glider1',
          'gone',
          'hawk',
          'heikki',
          'histoire',
          'hugh',
          'if6was9',
          'ingvar',
          'jan',
          'jedi',
          'jimi',
          'juhani',
          'khan',
          'lima',
          'midvale',
          'neko',
          'nesbit',
          'nexus6',
          'nisse',
          'notta1',
          'pam',
          'park',
          'pole',
          'pope',
          'pyro',
          'ram',
          'reliant',
          'rex',
          'rush',
          'seoul',
          'skip',
          'stan',
          'sue',
          'suzy',
          'tab',
          'testi',
          'thelorax',
          'tika',
          'tnt',
          'toto1',
          'tre',
          'wind',
          'x-men',
          'xyz',
          'zxc',
          '369',
          'Abcdef',
          'Asdfgh',
          'Changeme',
          'NCC1701',
          'Zxcvbnm',
          'demo',
          'doom2',
          'e',
          'good-luck',
          'homebrew',
          'm1911a1',
          'nat',
          'ne1410s',
          'ne14a69',
          'zhongguo',
          'sample123',
          '0852',
          'basf',
          'OU812',
          '!@#$%',
          'informix',
          'majordomo',
          'news',
          'temp',
          'trek',
          '!@#$%^',
          '!@#$%^&*',
          'Pentium',
          'Raistlin',
          'adi',
          'bmw',
          'law',
          'm',
          'new',
          'opus',
          'plus',
          'visa',
          'www',
          'y',
          'zzz',
          '1332',
          '1950',
          '3141',
          '3533',
          '4055',
          '4854',
          '6301',
          'Bonzo',
          'ChangeMe',
          'Front242',
          'Gretel',
          'Michel1',
          'Noriko',
          'Sidekick',
          'Sverige',
          'Swoosh',
          'Woodrow',
          'aa',
          'ayelet',
          'barn',
          'betacam',
          'biz',
          'boat',
          'cuda',
          'doc',
          'hal',
          'hallowell',
          'haro',
          'hosehead',
          'i',
          'ilmari',
          'irmeli',
          'j1l2t3',
          'jer',
          'kcin',
          'kerrya',
          'kissa2',
          'leaf',
          'lissabon',
          'mart',
          'matti1',
          'mech',
          'morecats',
          'paagal',
          'performa',
          'prof',
          'ratio',
          'ship',
          'slip',
          'stivers',
          'tapani',
          'targas',
          'test2',
          'test3',
          'tula',
          'unix',
          'user1',
          'xanth',
          '!@#$%^&',
          '1701d',
          '@#$%^&',
          'Qwert',
          'allo',
          'dirk',
          'go',
          'newcourt',
          'nite',
          'notused',
          'sss']

def Decrypt(data):
    if sys.version_info[0] == 2:
        data = [ord(char) for char in data]
    seed = data[0]
    version = data[1] ^ seed
    projectkey = data[2] ^ seed
    ignore = int((seed & 6) / 2)

    result = [' seed: 0x%02x' % seed, ' version: 0x%02x' % version, ' projectkey: 0x%02x' % projectkey, ' ignore: %d' % ignore]

    pb = projectkey
    decoded = []
    for iter in range(3, len(data)):
        byte = ((data[iter - 2] + pb) ^ data[iter]) % 0x100
        if ignore == 0:
            decoded.append(byte)
        else:
           ignore -= 1
        pb = byte

    return result, decoded

def IntegersToHex(integers):
    return ''.join(['%02x' % integer for integer in integers])

def IntegerToBinary(value, bits):
    binary = bin(value)[2:]
    return '0' * (bits - len(binary)) + binary

def IsBitSet(value, bitPosition):
    mask = 2 ** bitPosition
    return value & mask == mask

class cVBAProject(cPluginParent):

    macroOnly = False

    name = 'VBA project plugin'

    def __init__(self, name, stream, options):
        self.streamname = name
        self.stream = stream
        self.options = options

        self.ran = False

    def Analyze(self):
        result = []

        if self.streamname[-1] != 'PROJECT':
            return result

        oMatch = re.search(b'DPB="([A-F0-9]+)"', self.stream, re.I)
        if oMatch == None:
            return result

        oParser = optparse.OptionParser()
        oParser.add_option('-w', '--wordlist', type=str, default='', help='Try to crack password with provided passwordlist')
        (options, args) = oParser.parse_args(self.options.split(' '))

        encoded = binascii.a2b_hex(oMatch.groups()[0])
        result, decoded = Decrypt(encoded)

        result.insert(0, 'DPB="%s" decodes to:' % oMatch.groups()[0].decode())
        if decoded[0:4] == [29, 0, 0, 0]:
            data = []
            for index, value in enumerate(IntegerToBinary(decoded[5] * 0x10000 + decoded[6] * 0x100 + decoded[7],24)):
                data.append(decoded[8 + index] if value == '1' else 0x00)
            result.append(' VBA project is password protected')
            extractedsha1 = IntegersToHex(data[4:24])
            extractedsalt = IntegersToHex(data[0:4])
            result.append('  JtR hash: vbapassword:$dynamic_24$%s$HEX$%s' % (extractedsha1, extractedsalt))
            result.append('  Hashcat hash (-m 110 --hex-salt): %s:%s' % (extractedsha1, extractedsalt))

            if options.wordlist == '':
                filename = '.'
            else:
                filename = options.wordlist
            if sys.version_info[0] == 2:
                salt = ''.join([chr(byte) for byte in data[0:4]])
            else:
                salt = bytes(data[0:4])
            for password in GetDictionary(filename):
                try:
                    calculatedsha1 = hashlib.sha1(password.encode(encoding='utf-8', errors='ignore') + salt).hexdigest()
                except UnicodeDecodeError:
                    calculatedsha1 = ''
                if calculatedsha1 == extractedsha1:
                    result.append('  Password: %s' % password)
                    break

        elif decoded == [1, 0, 0, 0, 0]:
            result.append(' VBA project is not password protected')
        else:
            length = struct.unpack('<I', bytes(decoded)[0:4])[0]
            plaintext = bytes(decoded)[4:]
            if length == len(plaintext):
                result.append(' Plaintext password: ' + plaintext.rstrip(b'\x00').decode())
            else:
                result.append(' Unexpected data: ' + repr(decoded))

        self.ran = True

        oMatch = re.search(b'CMG="([A-F0-9]+)"', self.stream, re.I)
        if oMatch != None:
            encoded = binascii.a2b_hex(oMatch.groups()[0])
            resultCMG, decodedCMG = Decrypt(encoded)
            result.append('CMG="%s" decodes to:' % oMatch.groups()[0].decode())
            result.extend(resultCMG)
            if len(decodedCMG) != 8 or decodedCMG[0:4] != [4, 0, 0, 0]:
                result.append(' Unexpected data: ' + repr(decodedCMG))
            else:
                projectProtectionState = struct.unpack('<I', bytes(decodedCMG)[4:8])[0]
                result.append(' ProjectProtectionState: 0x%08x' % projectProtectionState)
                result.append('  fUserProtected: %s' % IsBitSet(projectProtectionState, 0))
                result.append('  fHostProtected: %s' % IsBitSet(projectProtectionState, 1))
                result.append('  fVBEProtected:  %s' % IsBitSet(projectProtectionState, 2))

        oMatch = re.search(b'GC="([A-F0-9]+)"', self.stream, re.I)
        if oMatch != None:
            encoded = binascii.a2b_hex(oMatch.groups()[0])
            resultGC, decodedGC = Decrypt(encoded)
            result.append('GC="%s" decodes to:' % oMatch.groups()[0].decode())
            result.extend(resultGC)
            if len(decodedGC) != 5 or decodedGC[0:4] != [1, 0, 0, 0]:
                result.append(' Unexpected data: ' + repr(decodedGC))
            else:
                projectVisibilityState = struct.unpack('<B', bytes(decodedGC)[4:5])[0]
                result.append(' ProjectVisibilityState: 0x%02x %s' % (projectVisibilityState, {0x00: 'Not visible', 0xFF: 'Visible'}.get(projectVisibilityState, 'UNKNOWN')))

        return result

AddPlugin(cVBAProject)



#!/usr/bin/env python

__description__ = 'HTTP Heuristics plugin for oledump.py'
__author__ = 'Didier Stevens'
__version__ = '0.0.13'
__date__ = '2020/08/16'

"""

Source code put in public domain by Didier Stevens, no Copyright
https://DidierStevens.com
Use at your own risk

History:
  2014/11/12: start
  2014/11/13: added HTTP filter
  2014/11/14: added unencoded http string detection
  2014/11/15: changed name and plugin interface
  2014/11/21: changed interface: added options
  2014/12/12: added BruteforceDecode
  2015/02/02: 0.0.3 added base64
  2015/02/09: bugfix BruteforceDecode when empty string; added StringsPerLine
  2015/02/16: 0.0.4 added rot13
  2015/02/25: 0.0.5 joined lines ending with _ for Chr analysis
  2015/03/18: 0.0.6 also handle empty strings
  2015/03/23: 0.0.7 fixed regression bug Heuristics
  2015/04/01: 0.0.8 added PreProcess
  2016/12/11: 0.0.9 added iOffset loop
  2018/10/13: 0.0.10 changed XOR logic, added options (-e -k)
  2019/11/05: 0.0.11 Python 3 support
  2020/01/24: 0.0.12 added option -c
  2020/01/25: Python 3 bugfix; deduping of result
  2020/08/16: 0.0.13 added option -s

Todo:
"""

import re
import binascii
import codecs

def ReplaceFunction(match):
    try:
        return '(%d)' % eval(match.group(0))
    except:
        return match.group(0)

keywords = ['http:', 'https:']
extendedkeywords = ['msxml', 'adodb', 'shell', 'c:\\', 'cmd', 'powershell']

def StartsWithHTTP(str):
    tosearch = str.lower()
    for keyword in keywords:
        if tosearch.startswith(keyword):
            return True
    return False

def ContainsHTTP(str):
    tosearch = str.lower()
    for keyword in keywords:
        if keyword in tosearch:
            return True
    return False

class cHTTPHeuristics(cPluginParent):
    macroOnly = True
    name = 'HTTP Heuristics plugin'

    def __init__(self, name, stream, options):
        self.streamname = name
        self.streamOriginal = stream
        self.stream = stream
        self.options = options
        self.ran = False
        self.CheckFunction = StartsWithHTTP

    def Heuristics(self, data, noDecode=False):
        if self.CheckFunction(data):
            return data
        if self.CheckFunction(data[::-1]):
            return data[::-1]
        if noDecode:
            return data
        try:
            decoded = binascii.a2b_hex(data).decode()
            return self.Heuristics(decoded, True)
        except:
            if not re.compile(r'^[0-9a-zA-Z/+=]+$').match(data):
                return data
            try:
                decoded = binascii.a2b_base64(data).decode()
                return self.Heuristics(decoded, True)
            except:
                return data

    # bruteforce XOR; if we have more than 250 strings, split in short strings (< 10) = keys and long strings = ciphertext
    def BruteforceDecode(self, strings):
        ciphertexts = []
        keys = []
        result = []

        if len(strings) >= 250:
            for string1 in strings:
                if len(string1) >= 10:
                    ciphertexts.append(string1)
                else:
                    keys.append(string1)
        else:
            ciphertexts = strings
            keys = strings
        for key in keys:
            if key != '':
                for ciphertext in ciphertexts:
                    for iOffset in range(2):
                        cleartext = ''
                        for iIter in range(len(ciphertext)):
                            cleartext += chr(ord(ciphertext[iIter]) ^ ord(key[(iIter + iOffset)% len(key)]))
                        result.append(self.Heuristics(cleartext))

        return result

    def Strings(self):
        return re.compile(r'"([^"]+)"').findall(self.stream)

    # Concatenate all strings found on the same line
    def StringsPerLine(self):
        result = []
        oREString = re.compile(r'"([^"]*)"')

        for line in self.stream.split('\n'):
            stringsConcatenated = ''.join(oREString.findall(line))
            if stringsConcatenated != '':
                result.append(stringsConcatenated)

        return result

    def PreProcess(self, options):
        self.stream = re.sub(r'(\(\s*(\d+|\d+\.\d+)\s*[+*/-]\s*(\d+|\d+\.\d+)\s*\))', ReplaceFunction, self.streamOriginal)
        
        if options.space:
            self.stream = self.stream.replace(' ', '')

    def AnalyzeSub(self):
        global keywords

        oParser = optparse.OptionParser()
        oParser.add_option('-e', '--extended', action='store_true', default=False, help='Use extended keywords')
        oParser.add_option('-k', '--keywords', type=str, default='', help='Provide keywords (separator is ,)')
        oParser.add_option('-c', '--contains', action='store_true', default=False, help='Check if string contains keyword')
        oParser.add_option('-s', '--space', action='store_true', default=False, help='Ignore space characters')
        (options, args) = oParser.parse_args(self.options.split(' '))

        self.PreProcess(options)

        if options.extended:
            keywords = keywords + extendedkeywords

        if options.keywords != '':
            keywords = options.keywords.split(',')

        if options.contains:
            self.CheckFunction = ContainsHTTP

        result = []

        oREChr = re.compile(r'((chr[w\$]?\(\d+\)(\s*&\s*)?)+)', re.IGNORECASE)
        oREDigits = re.compile(r'\d+')
        for foundTuple in oREChr.findall(self.stream.replace('_\r\n', '')):
            chrString = ''.join(map(lambda x: chr(int(x)), oREDigits.findall(foundTuple[0])))
            if chrString != '':
                result.append(self.Heuristics(chrString))

        oREHexBase64 = re.compile(r'"([0-9a-zA-Z/+=]+)"')
        for foundString in oREHexBase64.findall(self.stream):
            if foundString != '':
                    result.append(self.Heuristics(foundString))

        oREHTTP = re.compile(r'"(http[^"]+)"')
        for foundString in oREHTTP.findall(self.stream):
            if foundString != '':
                    result.append(foundString)

        resultHttp = [line for line in result if self.CheckFunction(line)]

        if resultHttp == []:
            resultHttp = [line for line in self.BruteforceDecode(result) if self.CheckFunction(line)]

        if resultHttp == []:
            resultHttp = [codecs.encode(line, 'rot-13') for line in self.Strings() if ContainsHTTP(codecs.encode(line, 'rot-13'))]
        else:
            return resultHttp

        if resultHttp == []:
            resultHttp = [line for line in self.StringsPerLine() if ContainsHTTP(line)]
        else:
            return resultHttp

        if resultHttp == []:
            return result
        else:
            return resultHttp

    def Analyze(self):
        self.ran = True

        return set(self.AnalyzeSub())

AddPlugin(cHTTPHeuristics)



#!/usr/bin/env python

__description__ = 'msi plugin for oledump.py'
__author__ = 'Didier Stevens'
__version__ = '0.0.2'
__date__ = '2021/10/26'

"""

Source code put in public domain by Didier Stevens, no Copyright
https://DidierStevens.com
Use at your own risk

History:
  2018/02/18: start
  2021/10/26: 0.0.2 Python 3 fix

Todo:
"""

import binascii
import hashlib

#https://stackoverflow.com/questions/9734978/view-msi-strings-in-binary

def Convert(character):
    code = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz._!'
    number = ord(character)
    if number >= 0x3800 and number < 0x4800:
        return code[(number - 0x3800) & 0x3F] + code[((number - 0x3800) >> 6) & 0x3F]
    elif number >= 0x4800 and number <= 0x4840:
        return code[number - 0x4800]
    else:
        return character

class cMSI(cPluginParent):
    macroOnly = False
    name = 'msi plugin'

    def __init__(self, name, stream, options):
        self.streamname = name
        self.stream = stream
        self.options = options
        self.ran = False

    def Analyze(self):
        self.ran = True
        result = []
        result.append('%s %-16s %s' % ((repr('/'.join(map(lambda x: ''.join([Convert(y) for y in x]), self.streamname))) + ' ' * 100)[0:30], bytes([IFF(b >= 32 and b < 127, b, ord('.')) for b in self.stream[:16]]), hashlib.md5(self.stream).hexdigest()))

        return result

AddPlugin(cMSI)



#!/usr/bin/env python

__description__ = 'HTTP in Form /o plugin for oledump.py'
__author__ = 'Didier Stevens'
__version__ = '0.0.1'
__date__ = '2016/02/29'

"""

Source code put in public domain by Didier Stevens, no Copyright
https://DidierStevens.com
Use at your own risk

History:
  2016/02/29: start

Todo:
"""

import re

class cFO(cPluginParent):
    macroOnly = False
    name = 'HTTP in Form /o plugin'

    def __init__(self, name, stream, options):
        self.streamname = name
        self.stream = stream
        self.options = options
        self.ran = False

    def Analyze(self):
        result = []

        if len(self.streamname) > 1 and self.streamname[-1] == 'o':
            self.ran = True
            for match in re.findall(r'http[\x21-\x7E]+', self.stream):
                result.append(match)

        return result

AddPlugin(cFO)



#!/usr/bin/env python

__description__ = 'PowerPoint plugin for oledump.py'
__author__ = 'Didier Stevens'
__version__ = '0.0.3'
__date__ = '2025/03/04'

"""

Source code put in public domain by Didier Stevens, no Copyright
https://DidierStevens.com
Use at your own risk

History:
  2014/11/15: start
  2018/08/11: second start, fix zip.decompress with FlateDecode
  2022/04/26: 0.0.2 Python 3 fixes
  2025/03/04: 0.0.3 bugfix xbird0x

Todo:
"""

import struct
import re
import optparse

def CombineHexASCII(hexDump, asciiDump, length):
    if hexDump == '':
        return ''
    return hexDump + '  ' + (' ' * (3 * (length - len(asciiDump)))) + asciiDump

def HexASCII(data, length=16):
    result = []
    if len(data) > 0:
        hexDump = ''
        asciiDump = ''
        for i, b in enumerate(data):
            if i % length == 0:
                if hexDump != '':
                    result.append(CombineHexASCII(hexDump, asciiDump, length))
                hexDump = '%08X:' % i
                asciiDump = ''
            hexDump += ' %02X' % b
            asciiDump += IFF(b >= 32, chr(b), '.')
        result.append(CombineHexASCII(hexDump, asciiDump, length))
    return result

def FlateDecode(data):
    oDecompress = zlib.decompressobj()
    return oDecompress.decompress(data)

class cPPT(cPluginParent):
    macroOnly = False
    name = 'PowerPoint plugin'

    def __init__(self, name, stream, options):
        self.streamname = name
        self.stream = stream
        self.options = options
        self.ran = False

    def AnalyzeSub(self, stream, indent, options):
        while stream != b'':
            formatcodes = '<HHI'
            formatsize = struct.calcsize(formatcodes)
            recVerInstance, recType, recLen = struct.unpack(formatcodes, stream[0:formatsize])
            container = recType in [0x2f14, 0x03e8, 0x0fc9, 0x03f0, 0x03ff, 0x03ee, 0x03f8, 0x0409, 0x1006, 0x100e, 0x0fee, 0x0fd7, 0x1007, 0x100d, 0x0fcc, 0x0fce, 0x100f, 0x1010]
            recInstance = recVerInstance // 0x10
            recVer = recVerInstance % 0x10
            stream = stream[formatsize:]
            data = stream[:recLen]
            stream = stream[recLen:]

            if recType in self.drecTypes:
                recTypename = self.drecTypes[recType]
            else:
                recTypename = ''
            indicator = ' '
            if container:
                indicator = 'C'
            if recType == 0x1011:
                indicator = '!'
            line = '%s %3d %s%04x %02x %02x %6d %s ' % (indicator, self.counter, indent, recType, recVer, recInstance, recLen, recTypename)
            if options.select == '':
                self.result.append(line)
            elif options.select == str(self.counter):
                if options.hexascii:
                    self.result.extend(' ' + foundstring for foundstring in HexASCII(data, 8))
                elif options.extract:
                    if recType == 0x1011 and recVer == 0:
                        if recInstance == 1:
                            self.result = FlateDecode(data[4:])
                        else:
                            self.result = data
                else:
                    self.result.append(line)
            self.counter += 1

            if container:
                self.AnalyzeSub(data, indent + '  ', options)

    def Analyze(self):
        self.result = []

        #https://msdn.microsoft.com/en-us/library/dd945336(v=office.12).aspx
        self.drecTypes = {
            0x03E8: 'RT_Document',
            0x03E9: 'RT_DocumentAtom',
            0x03EA: 'RT_EndDocumentAtom',
            0x03EE: 'RT_Slide',
            0x03EF: 'RT_SlideAtom',
            0x03F0: 'RT_Notes',
            0x03F1: 'RT_NotesAtom',
            0x03F2: 'RT_Environment',
            0x03F3: 'RT_SlidePersistAtom',
            0x03F8: 'RT_MainMaster',
            0x03F9: 'RT_SlideShowSlideInfoAtom',
            0x03FA: 'RT_SlideViewInfo',
            0x03FB: 'RT_GuideAtom',
            0x03FD: 'RT_ViewInfoAtom',
            0x03FE: 'RT_SlideViewInfoAtom',
            0x03FF: 'RT_VbaInfo',
            0x0400: 'RT_VbaInfoAtom',
            0x0401: 'RT_SlideShowDocInfoAtom',
            0x0402: 'RT_Summary',
            0x0406: 'RT_DocRoutingSlipAtom',
            0x0407: 'RT_OutlineViewInfo',
            0x0408: 'RT_SorterViewInfo',
            0x0409: 'RT_ExternalObjectList',
            0x040A: 'RT_ExternalObjectListAtom',
            0x040B: 'RT_DrawingGroup',
            0x040C: 'RT_Drawing',
            0x040D: 'RT_GridSpacing10Atom',
            0x040E: 'RT_RoundTripTheme12Atom',
            0x040F: 'RT_RoundTripColorMapping12Atom',
            0x0410: 'RT_NamedShows',
            0x0411: 'RT_NamedShow',
            0x0412: 'RT_NamedShowSlidesAtom',
            0x0413: 'RT_NotesTextViewInfo9',
            0x0414: 'RT_NormalViewSetInfo9',
            0x0415: 'RT_NormalViewSetInfo9Atom',
            0x041C: 'RT_RoundTripOriginalMainMasterId12Atom',
            0x041D: 'RT_RoundTripCompositeMasterId12Atom',
            0x041E: 'RT_RoundTripContentMasterInfo12Atom',
            0x041F: 'RT_RoundTripShapeId12Atom',
            0x0420: 'RT_RoundTripHFPlaceholder12Atom',
            0x0422: 'RT_RoundTripContentMasterId12Atom',
            0x0423: 'RT_RoundTripOArtTextStyles12Atom',
            0x0424: 'RT_RoundTripHeaderFooterDefaults12Atom',
            0x0425: 'RT_RoundTripDocFlags12Atom',
            0x0426: 'RT_RoundTripShapeCheckSumForCL12Atom',
            0x0427: 'RT_RoundTripNotesMasterTextStyles12Atom',
            0x0428: 'RT_RoundTripCustomTableStyles12Atom',
            0x07D0: 'RT_List',
            0x07D5: 'RT_FontCollection',
            0x07D6: 'RT_FontCollection10',
            0x07E3: 'RT_BookmarkCollection',
            0x07E4: 'RT_SoundCollection',
            0x07E5: 'RT_SoundCollectionAtom',
            0x07E6: 'RT_Sound',
            0x07E7: 'RT_SoundDataBlob',
            0x07E9: 'RT_BookmarkSeedAtom',
            0x07F0: 'RT_ColorSchemeAtom',
            0x07F8: 'RT_BlipCollection9',
            0x07F9: 'RT_BlipEntity9Atom',
            0x0BC1: 'RT_ExternalObjectRefAtom',
            0x0BC3: 'RT_PlaceholderAtom',
            0x0BDB: 'RT_ShapeAtom',
            0x0BDC: 'RT_ShapeFlags10Atom',
            0x0BDD: 'RT_RoundTripNewPlaceholderId12Atom',
            0x0F9E: 'RT_OutlineTextRefAtom',
            0x0F9F: 'RT_TextHeaderAtom',
            0x0FA0: 'RT_TextCharsAtom',
            0x0FA1: 'RT_StyleTextPropAtom',
            0x0FA2: 'RT_MasterTextPropAtom',
            0x0FA3: 'RT_TextMasterStyleAtom',
            0x0FA4: 'RT_TextCharFormatExceptionAtom',
            0x0FA5: 'RT_TextParagraphFormatExceptionAtom',
            0x0FA6: 'RT_TextRulerAtom',
            0x0FA7: 'RT_TextBookmarkAtom',
            0x0FA8: 'RT_TextBytesAtom',
            0x0FA9: 'RT_TextSpecialInfoDefaultAtom',
            0x0FAA: 'RT_TextSpecialInfoAtom',
            0x0FAB: 'RT_DefaultRulerAtom',
            0x0FAC: 'RT_StyleTextProp9Atom',
            0x0FAD: 'RT_TextMasterStyle9Atom',
            0x0FAE: 'RT_OutlineTextProps9',
            0x0FAF: 'RT_OutlineTextPropsHeader9Atom',
            0x0FB0: 'RT_TextDefaults9Atom',
            0x0FB1: 'RT_StyleTextProp10Atom',
            0x0FB2: 'RT_TextMasterStyle10Atom',
            0x0FB3: 'RT_OutlineTextProps10',
            0x0FB4: 'RT_TextDefaults10Atom',
            0x0FB5: 'RT_OutlineTextProps11',
            0x0FB6: 'RT_StyleTextProp11Atom',
            0x0FB7: 'RT_FontEntityAtom',
            0x0FB8: 'RT_FontEmbedDataBlob',
            0x0FBA: 'RT_CString',
            0x0FC1: 'RT_MetaFile',
            0x0FC3: 'RT_ExternalOleObjectAtom',
            0x0FC8: 'RT_Kinsoku',
            0x0FC9: 'RT_Handout',
            0x0FCC: 'RT_ExternalOleEmbed',
            0x0FCD: 'RT_ExternalOleEmbedAtom',
            0x0FCE: 'RT_ExternalOleLink',
            0x0FD0: 'RT_BookmarkEntityAtom',
            0x0FD1: 'RT_ExternalOleLinkAtom',
            0x0FD2: 'RT_KinsokuAtom',
            0x0FD3: 'RT_ExternalHyperlinkAtom',
            0x0FD7: 'RT_ExternalHyperlink',
            0x0FD8: 'RT_SlideNumberMetaCharAtom',
            0x0FD9: 'RT_HeadersFooters',
            0x0FDA: 'RT_HeadersFootersAtom',
            0x0FDF: 'RT_TextInteractiveInfoAtom',
            0x0FE4: 'RT_ExternalHyperlink9',
            0x0FE7: 'RT_RecolorInfoAtom',
            0x0FEE: 'RT_ExternalOleControl',
            0x0FF0: 'RT_SlideListWithText',
            0x0FF1: 'RT_AnimationInfoAtom',
            0x0FF2: 'RT_InteractiveInfo',
            0x0FF3: 'RT_InteractiveInfoAtom',
            0x0FF5: 'RT_UserEditAtom',
            0x0FF6: 'RT_CurrentUserAtom',
            0x0FF7: 'RT_DateTimeMetaCharAtom',
            0x0FF8: 'RT_GenericDateMetaCharAtom',
            0x0FF9: 'RT_HeaderMetaCharAtom',
            0x0FFA: 'RT_FooterMetaCharAtom',
            0x0FFB: 'RT_ExternalOleControlAtom',
            0x1004: 'RT_ExternalMediaAtom',
            0x1005: 'RT_ExternalVideo',
            0x1006: 'RT_ExternalAviMovie',
            0x1007: 'RT_ExternalMciMovie',
            0x100D: 'RT_ExternalMidiAudio',
            0x100E: 'RT_ExternalCdAudio',
            0x100F: 'RT_ExternalWavAudioEmbedded',
            0x1010: 'RT_ExternalWavAudioLink',
            0x1011: 'RT_ExternalOleObjectStg',
            0x1012: 'RT_ExternalCdAudioAtom',
            0x1013: 'RT_ExternalWavAudioEmbeddedAtom',
            0x1014: 'RT_AnimationInfo',
            0x1015: 'RT_RtfDateTimeMetaCharAtom',
            0x1018: 'RT_ExternalHyperlinkFlagsAtom',
            0x1388: 'RT_ProgTags',
            0x1389: 'RT_ProgStringTag',
            0x138A: 'RT_ProgBinaryTag',
            0x138B: 'RT_BinaryTagDataBlob',
            0x1770: 'RT_PrintOptionsAtom',
            0x1772: 'RT_PersistDirectoryAtom',
            0x177A: 'RT_PresentationAdvisorFlags9Atom',
            0x177B: 'RT_HtmlDocInfo9Atom',
            0x177C: 'RT_HtmlPublishInfoAtom',
            0x177D: 'RT_HtmlPublishInfo9',
            0x177E: 'RT_BroadcastDocInfo9',
            0x177F: 'RT_BroadcastDocInfo9Atom',
            0x1784: 'RT_EnvelopeFlags9Atom',
            0x1785: 'RT_EnvelopeData9Atom',
            0x2AFB: 'RT_VisualShapeAtom',
            0x2B00: 'RT_HashCodeAtom',
            0x2B01: 'RT_VisualPageAtom',
            0x2B02: 'RT_BuildList',
            0x2B03: 'RT_BuildAtom',
            0x2B04: 'RT_ChartBuild',
            0x2B05: 'RT_ChartBuildAtom',
            0x2B06: 'RT_DiagramBuild',
            0x2B07: 'RT_DiagramBuildAtom',
            0x2B08: 'RT_ParaBuild',
            0x2B09: 'RT_ParaBuildAtom',
            0x2B0A: 'RT_LevelInfoAtom',
            0x2B0B: 'RT_RoundTripAnimationAtom12Atom',
            0x2B0D: 'RT_RoundTripAnimationHashAtom12Atom',
            0x2EE0: 'RT_Comment10',
            0x2EE1: 'RT_Comment10Atom',
            0x2EE4: 'RT_CommentIndex10',
            0x2EE5: 'RT_CommentIndex10Atom',
            0x2EE6: 'RT_LinkedShape10Atom',
            0x2EE7: 'RT_LinkedSlide10Atom',
            0x2EEA: 'RT_SlideFlags10Atom',
            0x2EEB: 'RT_SlideTime10Atom',
            0x2EEC: 'RT_DiffTree10',
            0x2EED: 'RT_Diff10',
            0x2EEE: 'RT_Diff10Atom',
            0x2EEF: 'RT_SlideListTableSize10Atom',
            0x2EF0: 'RT_SlideListEntry10Atom',
            0x2EF1: 'RT_SlideListTable10',
            0x2F14: 'RT_CryptSession10Container',
            0x32C8: 'RT_FontEmbedFlags10Atom',
            0x36B0: 'RT_FilterPrivacyFlags10Atom',
            0x36B1: 'RT_DocToolbarStates10Atom',
            0x36B2: 'RT_PhotoAlbumInfo10Atom',
            0x36B3: 'RT_SmartTagStore11Container',
            0x3714: 'RT_RoundTripSlideSyncInfo12',
            0x3715: 'RT_RoundTripSlideSyncInfoAtom12',
            0xF125: 'RT_TimeConditionContainer',
            0xF127: 'RT_TimeNode',
            0xF128: 'RT_TimeCondition',
            0xF129: 'RT_TimeModifier',
            0xF12A: 'RT_TimeBehaviorContainer',
            0xF12B: 'RT_TimeAnimateBehaviorContainer',
            0xF12C: 'RT_TimeColorBehaviorContainer',
            0xF12D: 'RT_TimeEffectBehaviorContainer',
            0xF12E: 'RT_TimeMotionBehaviorContainer',
            0xF12F: 'RT_TimeRotationBehaviorContainer',
            0xF130: 'RT_TimeScaleBehaviorContainer',
            0xF131: 'RT_TimeSetBehaviorContainer',
            0xF132: 'RT_TimeCommandBehaviorContainer',
            0xF133: 'RT_TimeBehavior',
            0xF134: 'RT_TimeAnimateBehavior',
            0xF135: 'RT_TimeColorBehavior',
            0xF136: 'RT_TimeEffectBehavior',
            0xF137: 'RT_TimeMotionBehavior',
            0xF138: 'RT_TimeRotationBehavior',
            0xF139: 'RT_TimeScaleBehavior',
            0xF13A: 'RT_TimeSetBehavior',
            0xF13B: 'RT_TimeCommandBehavior',
            0xF13C: 'RT_TimeClientVisualElement',
            0xF13D: 'RT_TimePropertyList',
            0xF13E: 'RT_TimeVariantList',
            0xF13F: 'RT_TimeAnimationValueList',
            0xF140: 'RT_TimeIterateData',
            0xF141: 'RT_TimeSequenceData',
            0xF142: 'RT_TimeVariant',
            0xF143: 'RT_TimeAnimationValue',
            0xF144: 'RT_TimeExtTimeNodeContainer',
            0xF145: 'RT_TimeSubEffectContainer'
        }

        if self.streamname[-1] == 'PowerPoint Document':
            self.ran = True
            stream = self.stream

            oParser = optparse.OptionParser()
            oParser.add_option('-s', '--select', type=str, default='', help='Select record')
            oParser.add_option('-a', '--hexascii', action='store_true', default=False, help='Dump hex ascii')
            oParser.add_option('-e', '--extract', action='store_true', default=False, help='Extract object')
            (options, args) = oParser.parse_args(self.options.split(' '))

            self.counter = 1
            self.AnalyzeSub(stream, '', options)

        return self.result

AddPlugin(cPPT)



#!/usr/bin/env python

__description__ = 'Stream sample plugin for oledump.py'
__author__ = 'Didier Stevens'
__version__ = '0.0.1'
__date__ = '2016/09/05'

"""

Source code put in public domain by Didier Stevens, no Copyright
https://DidierStevens.com
Use at your own risk

History:
  2016/09/05: start

Todo:
"""

# A plugin defines a class that is instantiated for each stream in the OLE document.
# This class inherits from class cPluginParent.
# The name of the class (cStreamSample in this example) can be chosen freely.
# But avoid name conflicts with other plugins.
class cStreamSample(cPluginParent):

    # Set class variable macroOnly to True if the plugins requires VBA macro text (decompressed stream),
    # or to False if the plugin requires the raw stream content.
    # This class variable is defined and set to False in the parent class cPluginParent.
    macroOnly = False

    # Class variable name contains the name of the plugin to be displayed in oledump's output.
    name = 'Stream sample plugin'

    # Method __init__ provides data via 3 arguments:
    #  name is the name of the stream as returned by olefile's listdir method.
    #  stream is the content of the stream as returned by olefile's read method.
    #   stream is None is no stream is present (container)
    #   if class variable macroOnly is True, then stream contains the VBA macro code as one long string (e.g. the decompressed stream)
    #  options is a string passed to --pluginoptions.
    def __init__(self, name, stream, options):
        # Storing the arguments for later use by Analyze method
        self.streamname = name
        self.stream = stream
        self.options = options

        # Object property ran must be set to False before the plugin runs.
        # oledump only displays output for the plugin if property ran is True.
        self.ran = False

    # Method Analyze is called by oledump to let the plugin analyze the stream.
    # This method must return a list of strings: this is the plugin output to be displayed by oledump.
    # This method must also set object property ran to True to have oledump display output for this plugin.
    def Analyze(self):

        # This example counts the numbers of bytes with value FF in the stream,
        # and produces one line of output if the count is more than 0.
        result = []
        ffs = [b for b in self.stream if b == '\xff']
        if len(ffs) > 0:
            result.append('Number of 0xFF bytes in stream: %d' % len(ffs))

        self.ran = True

        return result

# The plugin class must be registered with oledump by calling function AddPlugin with
# the name of the plugin class (cStreamSample in this example).
AddPlugin(cStreamSample)

# The plugin code is not restricted or sandboxed. Plugin code runs with the same privileges and accesses as oledump.
# The plugin code and interface is not validated before execution.
# There is no object persistence across streams. If you need this, get in touch.



#!/usr/bin/env python

__description__ = 'VBA declare/createobject plugin for oledump.py'
__author__ = 'Didier Stevens'
__version__ = '0.0.5'
__date__ = '2022/07/21'

"""

Source code put in public domain by Didier Stevens, no Copyright
https://DidierStevens.com
Use at your own risk

History:
  2015/10/25: start
  2015/11/04: added keywords
  2019/11/25: 0.0.3 update for GetObject, Callbyname and Shell
  2022/07/21: 0.0.4 added generalization
  2022/09/04: 0.0.5 improved generalization and added option -p

Todo:
"""

import re

#https://github.com/decalage2/oletools/blob/f2cbbbaea5e8a809360ab338bd54ae281e4fd54f/oletools/olevba.py
RESERVED_KEYWORDS = ['AutoExec', 'AutoOpen', 'DocumentOpen', 'AutoExit', 'AutoClose', 'Document_Close', 'DocumentBeforeClose', 'DocumentChange', 'AutoNew', 'Document_New', 'NewDocument', 'Document_Open', 'Document_BeforeClose', 'Auto_Open', 'Workbook_Open', 'Workbook_Activate', 'Auto_Ope', 'Auto_Close', 'Workbook_Close', 'Workbook_BeforeClose', 'Worksheet_Calculate']

def IsReservedKeyword(keyword):
    return keyword.lower() in [reserved.lower() for reserved in RESERVED_KEYWORDS]

def ContainsString(stringsToFind, containingString):
    containingString = containingString.lower()
    for stringToFind in stringsToFind:
        if stringToFind.lower() in containingString:
            return True
    return False
            
def ExtractDeclareFunctionSub(line):
    if ContainsString(['declare'], line):
        oMatch = re.search(r'(function|sub)\s+(\S+)\s+', line, re.I)
        if oMatch == None:
            return None
        keyword = oMatch.group(2)
        oMatch = re.search(r'\s+alias\s+"([^"]+)"', line, re.I)
        if oMatch == None:
            return keyword, None
        else:
            return keyword, oMatch.group(1)
    else:
        return None, None

def ExtractSetObjectVariable(line):
    if ContainsString(['createobject', 'getobject'], line):
        oMatch = re.search(r'(\S+)\s*=\s*(CreateObject|GetObject)', line, re.I)
        if oMatch == None:
            return None
        return oMatch.group(1)
    else:
        return None

def SpecialTrim(input):
    unwanteds = ['(', ',']
    for unwanted in unwanteds:
        position = input.find(unwanted)
        if position == -1:
            continue
        input = input[:position]
    return input

def ExtractDimVariable(line):
    if ContainsString(['dim'], line):
        oMatch = re.search(r'\s*Dim\s+(\S+)', line, re.I)
        if oMatch == None:
            return None
        variableName = SpecialTrim(oMatch.group(1))
        return variableName
    else:
        return None

def ExtractSubVariable(line):
    if ContainsString(['sub', 'function'], line):
        oMatch = re.search(r'\s*(sub|function)\s+(\S+)', line, re.I)
        if oMatch == None:
            return []
        result = [oMatch.group(2).split('(')[0]]
        if not '(' in line:
            return result
        arguments = line.split('(')[1]
        if not ')' in arguments:
            return result
        arguments = arguments.split(')')[0]
        for argument in arguments.split(','):
            argument = argument.strip()
            if argument.lower().startswith('byref ') or argument.lower().startswith('byval '):
                argument = argument[6:] 
            oMatch = re.search(r'\s*(\S+)', argument, re.I)
            if oMatch != None:
                result.append(oMatch.group(1))
        return result
    else:
        return []

def ExtractVariableEqual(line):
    if ContainsString(['='], line):
        oMatch = re.search(r'\s*(\S+)\s*=', line, re.I)
        if oMatch == None:
            return None
        variableName = SpecialTrim(oMatch.group(1))
        return variableName
    else:
        return None

def ExtractMoreVariables(line):
    result = ExtractDimVariable(line)
    if result != None:
        return [result]
    result = ExtractSubVariable(line)
    if result != []:
        return result
    result = ExtractVariableEqual(line)
    if result != None:
        return [result]
    return []

def ReplaceOutsideStrings(line, search, replace):
    # add code to handle "" inside string
    result = []
    outsideString = ''
    insideString = ''
    for character in line:
        if outsideString == '':
            if character == '"':
                outsideString += character
                result.append(insideString)
                insideString = ''
            else:
                insideString += character
        else:
            if character == '"':
                outsideString += character
                result.append(outsideString)
                outsideString = ''
            else:
                outsideString += character
    if insideString != '':
        result.append(insideString)
    if outsideString != '':
        result.append(outsideString)

    newline = ''
    for index, value in enumerate(result):
        if index % 2 == 0:
            value = re.sub(r'\b%s\b' % search, replace, value)
        newline += value
    return newline

class cVBADCO(cPluginParent):
    macroOnly = True
    name = 'VBA DCO (Declare/CreateObject) plugin'

    def __init__(self, name, stream, options):
        self.streamname = name
        self.stream = stream
        self.options = options
        self.ran = False

    def Analyze(self):
        oParser = optparse.OptionParser()
        oParser.add_option('-g', '--generalize', action='store_true', default=False, help='Generalize identifiers')
        oParser.add_option('-a', '--all', action='store_true', default=False, help='Output all lines when option -g is used')
        oParser.add_option('-p', '--predefine', default='', help='Predefine identifiers')
        (options, args) = oParser.parse_args(self.options.split(' '))

        self.ran = True

        dAliases = {}
        dPredefinitions = {}
        for predefinition in options.predefine.split(','):
            if predefinition == '':
                continue
            key, value = predefinition.split(':', 1)
            dPredefinitions[int(key)] = value

        oREDCO = re.compile(r'\b(declare|createobject|getobject|callbyname|shell)\b', re.I)
        result = [line.strip() for line in self.stream.split('\n') if re.search(oREDCO, line) != None]
        keywords = []
        for line in result:
            keyword, alias = ExtractDeclareFunctionSub(line)
            if keyword != None and not keyword in keywords and not IsReservedKeyword(keyword):
                keywords.append(keyword)
            if alias != None:
                dAliases[keyword] = alias
            keyword = ExtractSetObjectVariable(line)
            if keyword != None and not keyword in keywords and not IsReservedKeyword(keyword):
                keywords.append(keyword)
        keywordLines = [line.strip() for line in self.stream.split('\n') if ContainsString(keywords, line)]
        if keywordLines != []:
            result.append('-' * 80)
            result.extend(keywordLines)

        if not options.generalize:
            return result

        dKeywords = {}
        for index, keyword in enumerate(keywords):
            index = index + 1
            if keyword in dAliases:
                dKeywords[keyword] = dAliases[keyword]
            else:
                dKeywords[keyword] = dPredefinitions.get(index, 'Identifier%04d' % (index))
        for line in self.stream.split('\n'):
            for variable in ExtractMoreVariables(line):
                if not variable in dKeywords and not IsReservedKeyword(variable):
                    index = len(dKeywords) + 1
                    dKeywords[variable] = dPredefinitions.get(index, 'Identifier%04d' % (index))
                
        if options.all:
            toprocess = self.stream.split('\n')
        else:
            toprocess = result
        result = []
        for line in toprocess:
            for key, value in sorted(dKeywords.items(), key=lambda x: len(x[0]), reverse=True):
                line = ReplaceOutsideStrings(line, key, value)
            result.append(line)
        return result

AddPlugin(cVBADCO)
